<?php
/**
* BotReplyEngine.php - Main component file
*
* This file is part of the BotReply component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\BotReply;

use Illuminate\Support\Arr;
use Illuminate\Validation\Rule;
use App\Yantrana\Base\BaseEngine;
use Illuminate\Database\Query\Builder;
use App\Yantrana\Components\BotReply\Repositories\BotReplyRepository;
use App\Yantrana\Components\BotReply\Interfaces\BotReplyEngineInterface;
use App\Yantrana\Components\Contact\Repositories\ContactCustomFieldRepository;

class BotReplyEngine extends BaseEngine implements BotReplyEngineInterface
{
    /**
     * @var  BotReplyRepository $botReplyRepository - BotReply Repository
     */
    protected $botReplyRepository;

    /**
     * @var  ContactCustomFieldRepository $contactCustomFieldRepository - ContactCustomField Repository
     */
    protected $contactCustomFieldRepository;

    /**
      * Constructor
      *
      * @param  BotReplyRepository $botReplyRepository - BotReply Repository
      *
      * @return  void
      *-----------------------------------------------------------------------*/

    public function __construct(
        BotReplyRepository $botReplyRepository,
        ContactCustomFieldRepository $contactCustomFieldRepository,
        )
    {
        $this->botReplyRepository = $botReplyRepository;
        $this->contactCustomFieldRepository = $contactCustomFieldRepository;
    }

    public function preDataForBots() {
        $vendorId = getVendorId();

        $dynamicFieldsToReplace = [
            '{first_name}',
            '{last_name}',
            '{full_name}',
            '{phone_number}',
            '{country}',
        ];

        $customFields = $this->contactCustomFieldRepository->fetchItAll([
            'vendors__id' => $vendorId
        ]);

        foreach ($customFields as $customField) {
            $dynamicFieldsToReplace[] = "{{$customField->input_name}}";
        }

        return $this->engineSuccessResponse([
            'dynamicFields' => $dynamicFieldsToReplace
        ]);
    }

    /**
      * BotReply datatable source
      *
      * @return  array
      *---------------------------------------------------------------- */
    public function prepareBotReplyDataTableSource()
    {
        $botReplyCollection = $this->botReplyRepository->fetchBotReplyDataTableSource();
        // required columns for DataTables
        $requireColumns = [
            '_id',
            '_uid',
            'name',
            'reply_text',
            'trigger_type',
            'reply_trigger'
        ];
        // prepare data for the DataTables
        return $this->dataTableResponse($botReplyCollection, $requireColumns);
    }


    /**
      * BotReply delete process
      *
      * @param  mix $botReplyIdOrUid
      *
      * @return  array
      *---------------------------------------------------------------- */

    public function processBotReplyDelete($botReplyIdOrUid)
    {
        // fetch the record
        $botReply = $this->botReplyRepository->fetchIt($botReplyIdOrUid);
        // check if the record found
        if (__isEmpty($botReply)) {
            // if not found
            return $this->engineResponse(18, null, __tr('Bot Reply not found'));
        }
        // ask to delete the record
        if ($this->botReplyRepository->deleteIt($botReply)) {
            // if successful
            return $this->engineResponse(1, null, __tr('Bot Reply deleted successfully'));
        }
        // if failed to delete
        return $this->engineResponse(2, null, __tr('Failed to delete BotReply'));
    }

    /**
      * BotReply create
      *
      * @param  array $inputData
      *
      * @return  array
      *---------------------------------------------------------------- */

    public function processBotReplyCreate($inputData)
    {
        $vendorId = getVendorId();
        // check the feature limit
        $vendorPlanDetails = vendorPlanDetails('bot_replies', $this->botReplyRepository->countIt([
            'vendors__id' => $vendorId
        ]), $vendorId);
        if (!$vendorPlanDetails['is_limit_available']) {
            return $this->engineResponse(22, null, $vendorPlanDetails['message']);
        }
        $inputData['vendors__id'] = $vendorId;
        if(isset($inputData['trigger_type']) and ($inputData['trigger_type'] == 'welcome')) {
            // check if welcome message already exists
            if($this->botReplyRepository->countIt([
                'trigger_type' => 'welcome',
                'vendors__id' => $inputData['vendors__id']
            ])) {
                return $this->engineResponse(2, null, __tr('You can be only have 1 welcome message which you already used up.'));
            }
        }

        // ask to add record
        if ($this->botReplyRepository->storeBotReply($inputData)) {
            return $this->engineResponse(1, null, __tr('Bot Reply added.'));
        }

        return $this->engineResponse(2, null, __tr('Bot Reply not added.'));
    }

    /**
      * BotReply prepare update data
      *
      * @param  mix $botReplyIdOrUid
      *
      * @return  array
      *---------------------------------------------------------------- */

    public function prepareBotReplyUpdateData($botReplyIdOrUid)
    {
        $botReply = $this->botReplyRepository->fetchIt($botReplyIdOrUid);

        // Check if $botReply not exist then throw not found
        // exception
        if (__isEmpty($botReply)) {
            return $this->engineResponse(18, null, __tr('Bot Reply not found.'));
        }

        return $this->engineResponse(1, $botReply->toArray());
    }

    /**
      * BotReply process update
      *
      * @param  mixed $botReplyIdOrUid
      * @param  object $request
      *
      * @return  array
      *---------------------------------------------------------------- */

    public function processBotReplyUpdate($botReplyIdOrUid, $request)
    {
        $botReply = $this->botReplyRepository->fetchIt($botReplyIdOrUid);
        // Check if $botReply not exist then throw not found
        // exception
        if (__isEmpty($botReply)) {
            return $this->engineResponse(18, null, __tr('Bot Reply not found.'));
        }
        $vendorId = getVendorId();

        $triggerTypeValidations = [];
        if($botReply->trigger_type != 'welcome') {
            $triggerTypeValidations['reply_trigger'] = [
                "required",
                Rule::unique('bot_replies')->where(fn (Builder $query) => $query->where('vendors__id', $vendorId))->ignore($botReply->_id, '_id')
            ];
        }

        // validate for uniqueness
        $request->validate([
            "name" => [
                "required",
                Rule::unique('bot_replies')->where(fn (Builder $query) => $query->where('vendors__id', $vendorId))->ignore($botReply->_id, '_id')
            ],
            "reply_trigger" => $triggerTypeValidations,
        ]);

        $inputData = $request->all();
        $updateData = [
            'name' => $inputData['name'],
            'reply_text' => $inputData['reply_text'],
            'reply_trigger' => $request->reply_trigger,
        ];
        // Check if BotReply updated
        if ($this->botReplyRepository->updateIt($botReply, $updateData)) {
            return $this->engineResponse(1, null, __tr('Bot Reply updated.'));
        }

        return $this->engineResponse(14, null, __tr('Bot Reply not updated.'));
    }
}
