<?php
/**
* Contact.php - Model file
*
* This file is part of the Contact component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Contact\Models;

use App\Yantrana\Base\BaseModel;
use App\Yantrana\Components\WhatsAppService\Models\WhatsAppMessageLogModel;
use App\Yantrana\Support\Country\Models\Country;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Casts\Attribute;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasManyThrough;
use Illuminate\Database\Eloquent\Relations\HasOne;

class ContactModel extends BaseModel
{
    /**
     * @var string - The database table used by the model.
     */
    protected $table = 'contacts';

    /**
     * @var array - The attributes that should be casted to native types.
     */
    protected $casts = [
        'messaged_at' => 'datetime',
    ];

    /**
     * @var array - The attributes that are mass assignable.
     */
    protected $fillable = [
    ];

    protected $appends = [
        'full_name',
        'gravatar',
    ];

    protected $hidden = [
    ];

    public function groups(): HasManyThrough
    {
        return $this->hasManyThrough(ContactGroupModel::class, GroupContactModel::class, 'contacts__id', '_id', '_id', 'contact_groups__id');
    }

    public function customFieldWithValues(): HasManyThrough
    {
        return $this->hasManyThrough(ContactCustomFieldModel::class, ContactCustomFieldValueModel::class, 'contacts__id', '_id', '_id', 'contact_custom_fields__id')->with('userValue');
    }

    public function customFieldValues(): HasMany
    {
        return $this->hasMany(ContactCustomFieldValueModel::class, 'contacts__id', '_id', '_id', 'contact_custom_fields__id');
    }

    /**
     * prepare and get contact whatsapp number
     */
    protected function whatsappNumber(): Attribute
    {
        return Attribute::make(
            get: fn(mixed $value, array $attributes) => $attributes['wa_id'],
        );
    }

    /**
     * gravatar
     */
    protected function gravatar(): Attribute
    {
        return Attribute::make(
            get: fn(mixed $value, array $attributes) => 'https://www.gravatar.com/avatar/' . md5(strtolower(trim($attributes['email'] ?? ''))),
        );
    }

    /**
     * prepare and get contact full name
     */
    protected function fullName(): Attribute
    {
        return Attribute::make(
            get: fn(mixed $value, array $attributes) => (($attributes['first_name'] ?? '') . ' ' . ($attributes['last_name'] ?? '')),
        );
    }

    /**
     * Get the country
     */
    public function country(): BelongsTo
    {
        return $this->belongsTo(Country::class, 'countries__id', '_id');
    }

    /**
     * Get the country
     */
    public function lastMessage(): HasOne
    {
        return $this->hasOne(WhatsAppMessageLogModel::class, 'contacts__id', '_id')->latestOfMany('messaged_at');
    }

    /**
     * Get last incoming message
     */
    public function lastIncomingMessage(): HasOne
    {
        return $this->hasOne(WhatsAppMessageLogModel::class, 'contacts__id', '_id')->where([
            'is_incoming_message' => 1,
        ])->latest('messaged_at');
    }

    /**
     * Get last unread message
     */
    public function lastUnreadMessage(): HasOne
    {
        return $this->hasOne(WhatsAppMessageLogModel::class, 'contacts__id', '_id')->where([
            'status' => 'received',
            'is_incoming_message' => 1,
        ])->latestOfMany('messaged_at');
    }

    /**
     * Get last unread message
     */
    public function unreadMessages(): HasMany
    {
        return $this->hasMany(WhatsAppMessageLogModel::class, 'contacts__id', '_id')->where([
            'status' => 'received',
            'is_incoming_message' => 1,
        ]);
    }
}
