<?php
/**
* ContactRepository.php - Repository file
*
* This file is part of the Contact component.
*-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Contact\Repositories;

use App\Yantrana\Base\BaseRepository;
use App\Yantrana\Components\Contact\Interfaces\ContactRepositoryInterface;
use App\Yantrana\Components\Contact\Models\ContactModel;
use App\Yantrana\Support\Country\Models\Country;

class ContactRepository extends BaseRepository implements ContactRepositoryInterface
{
    /**
     * primary model instance
     *
     * @var object
     */
    protected $primaryModel = ContactModel::class;

    /**
     * Fetch contact datatable source
     *
     * @return mixed
     *---------------------------------------------------------------- */
    public function fetchContactDataTableSource($groupContactIds = [])
    {
        // basic configurations for dataTables data
        $dataTableConfig = [
            // searchable columns
            'searchable' => [
                'first_name',
                'last_name',
                'countries__id',
                'wa_id',
                'email',
            ],
        ];

        // get Model result for dataTables
        $query = $this->primaryModel::where([
            'vendors__id' => getVendorId()
        ]);
        if (!empty($groupContactIds)) {
            $query->whereIn('_id', $groupContactIds);
        }
        return $query->dataTables($dataTableConfig)->toArray();
    }

    /**
     * Delete $contact record and return response
     *
     * @param  object  $inputData
     * @return mixed
     *---------------------------------------------------------------- */
    public function deleteContact($contact)
    {
        // Check if $contact deleted
        if ($contact->deleteIt()) {
            // if deleted
            return true;
        }

        // if failed to delete
        return false;
    }

    /**
     * Store new contact record and return response
     *
     * @param  array  $inputData
     * @return mixed
     *---------------------------------------------------------------- */
    public function storeContact($inputData, $vendorId = null)
    {
        // prepare data to store
        $keyValues = [
            'first_name',
            'last_name',
            'countries__id' => $inputData['country'] ?? null,
            'email',
            'language_code',
            'wa_id' => $inputData['phone_number'],
            'vendors__id' => $vendorId ?: getVendorId(),
        ];

        return $this->storeIt($inputData, $keyValues);
    }

    public function getVendorContact(string|int $contactIdOrUid, ?string $vendorId = null)
    {
        $findBy = [
            'vendors__id' => $vendorId ? $vendorId : getVendorId(),
        ];
        if (is_numeric($contactIdOrUid)) {
            $findBy['_id'] = $contactIdOrUid;
        } else {
            $findBy['_uid'] = $contactIdOrUid;
        }

        return $this->with('customFieldValues')->fetchIt($findBy);
    }

    public function getVendorContactByWaId(int $waId, ?string $vendorId = null)
    {
        return $this->fetchIt([
            'vendors__id' => $vendorId ? $vendorId : getVendorId(),
            'wa_id' => $waId,
        ]);
    }

    public function getVendorContactWithUnreadDetails($contactUid = null, $vendorId = null)
    {
        $whereClause = [
            'vendors__id' => $vendorId ?: getVendorId(),
            // '_uid' => $contactUid,
        ];
        if($contactUid) {
            $whereClause['_uid'] = $contactUid;
        }
        return $this->primaryModel::where($whereClause)->with([
            'lastMessage',
            'lastUnreadMessage',
            'lastIncomingMessage',
        ])
            ->withCount('unreadMessages')
            ->first();
    }

    public function getVendorContactsWithUnreadDetails($vendorId = null)
    {
        return $this->primaryModel::where([
            'vendors__id' => $vendorId ?: getVendorId(),
        ])->with('lastMessage')
            ->withCount('unreadMessages')
            ->has('lastIncomingMessage')
            ->get();
    }
}
