<?php

/**
 * SubscriptionEngine.php - Main component file
 *
 * This file is part of the Subscription component.
 *-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Subscription;

use Throwable;
use Illuminate\Support\Arr;
use Illuminate\Support\Str;
use Laravel\Cashier\Cashier;
use App\Yantrana\Base\BaseEngine;
use Laravel\Cashier\Subscription;
use Stripe\Exception\AuthenticationException;
use Laravel\Cashier\Exceptions\IncompletePayment;
use App\Yantrana\Components\Vendor\Repositories\VendorRepository;
use App\Yantrana\Components\Subscription\Repositories\SubscriptionRepository;
use App\Yantrana\Components\Subscription\Interfaces\SubscriptionEngineInterface;

class SubscriptionEngine extends BaseEngine implements SubscriptionEngineInterface
{
    /**
     * @var SubscriptionRepository - Subscription Repository
     */
    protected $subscriptionRepository;

    /**
     * @var VendorRepository - Vendor Repository
     */
    protected $vendorRepository;

    /**
     * @var object - Holds subscriber eloquent object
     */
    protected $subscriber;

    /**
     * Constructor
     *
     * @param  SubscriptionRepository  $subscriptionRepository  - Subscription Repository
     * @return void
     *-----------------------------------------------------------------------*/
    public function __construct(SubscriptionRepository $subscriptionRepository, VendorRepository $vendorRepository)
    {
        $this->subscriptionRepository = $subscriptionRepository;
        $this->vendorRepository = $vendorRepository;
    }

    public function subscriber()
    {
        if (! __isEmpty($this->subscriber)) {
            return $this->subscriber;
        }
        $this->subscriber = $this->vendorRepository->fetchIt(getUserAuthInfo('vendor_id'));

        return $this->subscriber;
    }

    public function getCurrentPlan($planId = null)
    {
        // if plan id not sent search for current active subscription
        // to get the plan id
        if (! $planId) {
            $subscription = Subscription::query()->where(['vendor_model__id' => getVendorId()])->active()->first();
            if (__isEmpty($subscription)) {
                return null;
            }
            // if found active subscription
            $planId = $subscription->type;
        }
        $subscriptionPlans = getPaidPlans();
        if (__isEmpty($subscriptionPlans)) {
            return null;
        }
        $subscriptionPlan = Arr::get($subscriptionPlans, $planId);
        if (__isEmpty($subscriptionPlan)) {
            return null;
        }
        $hasPlan = Str::contains(json_encode($subscriptionPlan), $planId); // subscription->stripe_plan
        if ($hasPlan) {
            return $subscriptionPlan;
        }

        return null;
    }

    public function prepareData()
    {
        $isValidStripeKeys = false;
        if (getAppSettings('enable_stripe')) {
            if($this->validateStripeApiKey()) {
            $isValidStripeKeys = true;
            try {
                $stripeCustomer = $this->subscriber()->createOrGetStripeCustomer([
                    'name' => getUserAuthInfo('profile.full_name'),
                    'email' => getUserAuthInfo('profile.email'),
                    'address' => [
                        'line1' => getVendorSettings('address'),
                        'postal_code' => getVendorSettings('postal_code'),
                        'city' => getVendorSettings('city'),
                        'state' => getVendorSettings('state'),
                        'country' => getVendorSettings('country_code'),
                    ],
                ]);

            } catch (\Throwable $th) {

                //ALL SUBSCRIPTION MARK AS CANCELLED
                if ($this->getCurrentPlan() != null) {
                    $this->subscriber()->subscription($this->getCurrentPlan()['id'])->markAsCanceled();
                }

                //FORCEFULLY UPDATE VALUES
                $this->subscriber()->forceFill([
                    'stripe_id' => null,
                    'trial_ends_at' => null,
                    'pm_type' => null,
                    'pm_last_four' => null,
                ])->update();

                $this->prepareData();
            } catch (\Throwable $th) {
                return ['message' => $th->getMessage()];
            }
        }
    }
        $currentPlan = $this->getCurrentPlan();
        $currentSubscription = null;
        if (! __isEmpty($currentPlan)) {
            $currentSubscription = $this->subscriber()->subscription($currentPlan['id']);
        }
        $planSelectorId = null;
        if ($currentSubscription and (Arr::get($currentPlan, 'charges.monthly.price_id') === $currentSubscription->stripe_price)) {
            $planSelectorId = $currentPlan['id'].'___monthly';
        } elseif ($currentSubscription and (Arr::get($currentPlan, 'charges.yearly.price_id') === $currentSubscription->stripe_price)) {
            $planSelectorId = $currentPlan['id'].'___yearly';
        }
        $configPlans = getConfigPaidPlans();
        $dataToReturn = [
            'intent' => null,
            'currentPlan' => $currentPlan,
            'invoices' => [],
            'planSelectorId' => $planSelectorId,
            'currentSubscription' => $currentSubscription,
            'subscriber' => $this->subscriber(),
            'planDetails' => getPaidPlans(),
            'planStructure' => $configPlans,
            'freePlanDetails' => getFreePlan(),
            'freePlanStructure' => getConfigFreePlan(),
            'isValidStripeKeys' => $isValidStripeKeys,
        ];

        if (getAppSettings('enable_stripe') and $isValidStripeKeys) {
            $dataToReturn['intent'] = $this->subscriber()->createSetupIntent();
            $dataToReturn['invoices'] = $this->subscriber()->invoices();
        }

        return $dataToReturn;
    }

    public function processCancellation()
    {
        // request to
        try {
            $processedData = $this->subscriber()->subscription($this->getCurrentPlan()['id'])->cancel();
        } catch (Throwable $th) {

            return $this->engineFailedResponse([], $th->getMessage());
        }

        return $this->engineSuccessResponse([], __tr('Your subscription has been cancelled'));
    }

    public function processResume()
    {
        try {
            $processedData = $this->subscriber()->subscription($this->getCurrentPlan()['id'])->resume();
        } catch (Throwable $th) {

            return $this->engineFailedResponse([], $th->getMessage());
        }

        return $this->engineSuccessResponse([], __tr('Your subscription has been resumed'));
    }

    public function processChangePlan($request)
    {
        try {
            $planRequest = explode('___', $request->plan);
            $getPlanDetails = $this->getCurrentPlan($planRequest[0]);
            $planPriceId = Arr::get($getPlanDetails, 'charges.'.$planRequest[1].'.price_id');
            if (! $planPriceId) {
                return $this->engineFailedResponse([], __tr('Invalid Plan'));
            }

            $processedData = $this->subscriber()
                ->subscription(
                    $this->getCurrentPlan()['id']
                )->allowPaymentFailures()->swap($planPriceId);

            // In Laravel Cashier, when user change plans for a subscription, the local subscription name is not automatically updated. The subscription name in the database is typically set when the subscription is created and does not change automatically when the plan is updated.To update the local subscription name after changing the plan, Then our system can update the subscription record in your database.
            if (! __isEmpty($processedData)) {
                $processedData->update(['type' => $getPlanDetails['id']]);
            }
        } catch (IncompletePayment $exception) {
            return $this->engineResponse(21, [
                'redirect_to' => route(
                    'cashier.payment',
                    [$exception->payment->id, 'redirect' => route('subscription.read.show')]
                ),
            ]);
        } catch (Throwable $th) {
            return $this->engineFailedResponse([], $th->getMessage());
        }

        return $this->engineSuccessResponse([], __tr('Your subscription has been changed'));
    }

    public function processRedirectToBillingPortal()
    {
        return $this->subscriber()->redirectToBillingPortal(route('subscription.read.show'));
    }

    public function processDownloadInvoice($invoiceId)
    {
        return $this->subscriber()->downloadInvoice($invoiceId, [
            'vendor' => getVendorSettings('name'),
            'product' => getVendorSettings('name').__tr(' Subscription'),
        ]);
    }

    public function processCreate($request)
    {
        try {
            $planRequest = explode('___', $request->plan);
            $getPlanDetails = $this->getCurrentPlan($planRequest[0]);
            $planPriceId = Arr::get($getPlanDetails, 'charges.'.$planRequest[1].'.price_id');
            $trialDays = Arr::get($getPlanDetails, 'trial_days');
            $planId = Arr::get($getPlanDetails, 'id');
            $subscription = $this->subscriber()->newSubscription($planId, $planPriceId);
            if ($trialDays) {
                $subscription->trialDays($trialDays);
            }
            $subscription->allowPaymentFailures()->create($request->paymentMethod);
        } catch (IncompletePayment $exception) {
            return redirect()->route(
                'cashier.payment',
                [$exception->payment->id, 'redirect' => route('subscription.read.show')]
            );
        }

        return redirect(route('subscription.read.show'));
    }

    // use Laravel\Cashier\Cashier;
// use Stripe\Exception\AuthenticationException;

function validateStripeApiKey(): bool
{
    try {
        // Assuming you've set your Stripe keys in Cashier's configuration or .env
        $stripe = Cashier::stripe();

        // Attempt to retrieve Stripe account details
        $stripe->accounts->retrieve();

        return true; // If no exception is thrown, the API key is valid
    } catch (AuthenticationException $e) {
        return false; // Authentication with Stripe's API failed
    } catch (\Exception $e) {
        // Handle other potential exceptions
        return false;
    }
}


    /**
     * Delete all existing subscription entries.
     *
     * @return void
     */
    public function processDeleteSubscriptionEntries() {
        $this->subscriptionRepository->deleteItAll([
            [
                'id', '!=', null
            ]
        ]);
        return $this->engineSuccessResponse([], __tr('All the subscription entries has been cleared'));
    }

    public function prepareSubscriptionDataTableList()
    {
        $subscriptionCollection = $this->subscriptionRepository->fetchSubscriptionDataTableSource();
        $requireColumns = [
            'title',
            '_uid',
            'vendor_model__id',
            'plan_type',
            'stripe_id' => function ($keyItem) {
                if (__isEmpty($keyItem['stripe_id'])) {
                    return 'NA';
                }

                return $keyItem['stripe_id'];
            },
            'stripe_status',
            'stripe_price',
            // 'trial_ends_at',
            'ends_at' => function ($keyItem) {
                if (! ($keyItem['ends_at'])) {
                    return 'NA';
                } else {
                    return formatDateTime($keyItem['ends_at']);
                }
            },
            'created_at' => function ($keyItem) {
                if (__isEmpty($keyItem['created_at'])) {
                    return 'NA';
                }

                return formatDateTime($keyItem['created_at']);
            },
        ];

        return $this->dataTableResponse($subscriptionCollection, $requireColumns);
    }
}
