<?php

/**
 * VendorController.php - Controller file
 *
 * This file is part of the Vendor component.
 *-----------------------------------------------------------------------------*/

namespace App\Yantrana\Components\Vendor\Controllers;

use App\Yantrana\Base\BaseController;
use App\Yantrana\Components\Auth\AuthEngine;
use App\Yantrana\Components\Dashboard\DashboardEngine;
use App\Yantrana\Components\Vendor\VendorEngine;
use App\Yantrana\Support\CommonPostRequest;
use App\Yantrana\Support\CommonRequest;

class VendorController extends BaseController
{
    /**
     * @var VendorEngine - Vendor Engine
     */
    protected $vendorEngine;

    /**
     * @var AuthEngine - Auth Engine
     */
    protected $authEngine;

    /**
     * @var DashboardEngine - Dashboard Engine
     */
    protected $dashboardEngine;

    /**
     * Constructor
     *
     * @param  VendorEngine  $vendorEngine  - Vendor Engine
     * @return void
     *-----------------------------------------------------------------------*/
    public function __construct(VendorEngine $vendorEngine, AuthEngine $authEngine, DashboardEngine $dashboardEngine)
    {
        $this->vendorEngine = $vendorEngine;
        $this->authEngine = $authEngine;
        $this->dashboardEngine = $dashboardEngine;
    }

    /**
     * Manage User List.
     *
     * @return json object
     *---------------------------------------------------------------- */
    public function vendorDataTableList()
    {
        return $this->vendorEngine->prepareVendorDataTableList();
    }

    public function addVendor(CommonRequest $request)
    {
        $request->validate([
            'vendor_title' => 'required|string|min:2|max:100',
            'username' => 'required|string|alpha_dash|min:2|max:45|unique:users,username',
            'first_name' => 'required|string|min:1|max:45',
            'last_name' => 'required|string|min:1|max:45',
            'email' => 'required|string|email|max:255|unique:users,email',
            'password' => 'required|string|confirmed|min:8',
            'password_confirmation' => 'required',
        ]);
        $processReaction = $this->authEngine->processRegistration($request->all());

        return $this->processResponse($processReaction, [], [], true);
    }

    public function storeFront($vendorSlug)
    {
        return $this->loadView(
            'vendors.store-front',
            $this->vendorEngine->processStoreFront($vendorSlug)
        );
    }

    public function pwaManifest()
    {
        return response($this->loadView(
            'vendors.pwa-manifest'
        ))->header('Content-Type', 'application/manifest+json');
    }

    public function pwaServiceWorker()
    {
        return response($this->loadView(
            'vendors.pwa-service-worker-js'
        ))->header('Content-Type', 'text/javascript');
    }

    public function infoPage($vendorSlug, $pageSlug)
    {
        return $this->loadView(
            'vendors.info-page-view',
            $this->vendorEngine->pageInfo($pageSlug)
        );
    }

    /**
     * Prepare Vendor's Delete
     *
     * @param  mix  $vendorIdOrUid
     * @return json object
     */
    public function prepareVendorDelete($vendorIdOrUid)
    {
        // ask engine to process the request
        $processReaction = $this->vendorEngine->prepareVendorDelete($vendorIdOrUid);

        // get back to controller with engine response
        return $this->processResponse($processReaction, [], [], true);
    }

    /**
     * Prepare Update Vendor Data
     *
     * @param  mix  $vendorIdOrUid
     * @return json object
     */
    public function prepareUpdateVendorData($vendorIdOrUid)
    {
        // ask engine to process the request
        $processReaction = $this->vendorEngine->prepareVendorUpdateData($vendorIdOrUid);

        // get back to controller with engine response
        return $this->processResponse($processReaction, [], [], true);
    }

    /**
     * Update Vendor's Data
     *
     *
     * @return json object
     */
    public function updateVendorData(CommonRequest $request)
    {
        // process the validation based on the provided rules
        $request->validate([
            'vendorIdOrUid' => 'required',
            'userIdOrUid' => '',
            'title' => 'required|string|min:2|max:100',
            'username' => 'required|string|alpha_dash|min:2|max:45',
            'first_name' => 'required|string|min:1|max:45',
            'last_name' => 'required|string|min:1|max:45',
            'email' => 'required|string|email|max:255',
            'status' => '',
        ]);
        // ask engine to process the request
        $processReaction = $this->vendorEngine->processVendorUpdate($request->all());

        // get back with response
        return $this->processResponse($processReaction, [], [], true);
    }

    /**
     * Change Password Vendor Data
     *
     * @param  mix  $vendorIdOrUid
     * @return array
     */
    public function changePasswordVendorData($vendorIdOrUid)
    {
        // ask engine to process the request
        $processReaction = $this->vendorEngine->prepareVendorPasswordData($vendorIdOrUid);

        // get back to controller with engine response
        return $this->processResponse($processReaction, [], [], true);
    }

    /**
     * Change Password Vendor
     *
     *
     * @return json object
     */
    public function changePasswordVendor(CommonPostRequest $request)
    {
        $request->validate([
            // 'current_password' =>'required','different:old_password'
            'password' => ['required', 'min:6', 'confirmed'],
            'password_confirmation' => ['required', 'min:6', 'same:password'],
        ]);
        $processReaction = $this->vendorEngine->processChangePasswordBySuperAdmin($request->all());

        //check reaction code equal to 1
        return $this->processResponse($processReaction, [], [], true);
    }

    public function vendorDashboard($vendorIdOrUid)
    {
        $vendorInfo = $this->vendorEngine->getBasicSettings($vendorIdOrUid);

        return $this->loadView('vendors.vendor-dashboard', array_merge($this->dashboardEngine->prepareVendorDashboardData($vendorIdOrUid), [
            'vendorViewBySuperAdmin' => true,
            'vendorIdOrUid' => $vendorIdOrUid,
            'vendorInfo' => $vendorInfo,
            'vendorSlug' => $vendorInfo['slug'],
        ]));
    }

    public function vendorItems($vendorIdOrUid)
    {
        $vendorInfo = $this->vendorEngine->getBasicSettings($vendorIdOrUid);

        return $this->loadView('vendors.vendor-items', ['vendorIdOrUid' => $vendorIdOrUid, 'vendorInfo' => $vendorInfo]);
    }
}
